Program idserver;
{****************************************************************}
{*                                                              *}
{*             Zeljko Cvijanovic - Teslic                       *}
{*              E-mail: cvzeljko@gmail.com                      *}
{*       https://sites.google.com/site/zeljusorg/zeljusorg      *}
{****************************************************************}

{$mode objfpc}{$H+}

uses
{$IFDEF UNIX}//{$IFDEF UseCThreads}
cthreads,
{$ENDIF}//{$ENDIF}
SysUtils,
BaseUnix, MyServer;


Var
   { vars for daemonizing }
   bHup,
   bTerm : boolean;
   fLog : text;
   fPid : text;
   fTest : text;
   logname : string;
   pidname : string;
   aOld, aTerm, aHup : pSigActionRec;
   ps1 : psigset;
   sSet : cardinal;
   pid : pid_t;
   oldpid : pid_t;
   secs : longint;
   zerosigs : sigset_t;
   hr,mn,sc,sc100 : word;
   myserver1: THTTPServer;


{ handle SIGHUP & SIGTERM }
procedure DoSig(sig : longint);cdecl;
begin
     case sig of
     SIGHUP : bHup := true;
     SIGTERM : bTerm := true;
     end;
end;

{ open the log file }
Procedure NewLog;
Begin
     Assign(fLog,logname);
     Rewrite(fLog);
     Writeln(flog,'Log created at ',formatdatetime('hh:nn:ss',now));
     Close(fLog);
End;

{ open the log file }
Procedure SavePid(apid: integer);
Begin
     Assign(fPid,pidname);
     Rewrite(fPid);
     Writeln(fPid,apid);
     Close(fPid);
End;

{ save test file }
Procedure SaveTest(aname: string);
Begin
     Assign(fTest,'/tmp/'+aname);
     Rewrite(fTest);
     Writeln(fTest,'test: ',formatdatetime('hh:nn:ss',now));
     Close(fTest);
End;

{ open the log file }
Procedure LoadPid(var apid: integer);
var
   s: ansistring;
begin
  try
     Assign(fPid,pidname);
     Reset(fPid);
     Writeln(pidname);
     Read(fPid,s);
     Writeln(string(s));
     Close(fPid);
     apid := strtoint(s);
  except
     apid := 0;
  end;
end;

{ open the log file }
Procedure DeletePid();
begin
     Assign(fPid,pidname);
     Rewrite(fPid);
     Writeln(fPid,0);
     Close(fPid);
end;

begin
     writeln('Program: ',ParamStr(0));
     pidname := 'daemon.pid';
     oldpid := 0;
     LoadPid(oldpid);
     writeln('oldpid: ',oldpid);

     if oldpid=0 then begin

        logname := 'daemon.log';
        secs := 10;
        fpsigemptyset(zerosigs);

        { set global daemon booleans }
        bHup := true; { to open log file }
        bTerm := false;

        { block all signals except -HUP & -TERM }
        sSet := $ffffbffe;
        ps1 := @sSet;
        fpsigprocmask(sig_block,ps1,nil);

        { setup the signal handlers }
        new(aOld);
        new(aHup);
        new(aTerm);
        aTerm^.sa_handler{.sh} := SigactionHandler(@DoSig);

        aTerm^.sa_mask := zerosigs;
        aTerm^.sa_flags := 0;
        {$ifndef BSD} {Linux'ism}
        aTerm^.sa_restorer := nil;
        {$endif}
        aHup^.sa_handler := SigactionHandler(@DoSig);
        aHup^.sa_mask := zerosigs;
        aHup^.sa_flags := 0;
        {$ifndef BSD} {Linux'ism}
        aHup^.sa_restorer := nil;
        {$endif}
        fpSigAction(SIGTERM,aTerm,aOld);
        fpSigAction(SIGHUP,aHup,aOld);

        { daemonize }
        pid := fpFork;
        Case pid of
          0 : begin { we are in the child }
            Close(input); { close standard in }
            Close(output); { close standard out }
            Assign(output,'/dev/null');
            ReWrite(output);
            Close(stderr); { close standard error }
            Assign(stderr,'/dev/null');
            ReWrite(stderr);
          end;
          -1 : secs := 0; { forking error, so run as non-daemon }
           else begin
             SavePid(pid);
             Halt; { successful fork, so parent dies }
           end;
          end;

          { begin processing loop }
          repeat
                if bHup Then begin
                 {$I-}
                 Close(fLog);
                 {$I+}
                 IOResult;
                 NewLog;
                 myserver1:=THTTPServer.Create(nil); //this does not ...
                 myserver1.Active:= true;

                 bHup := false;
                end;
                {----------------------}

                { Do your daemon stuff }
                Append(flog);
                Writeln(flog,'daemon code activated at ',formatdatetime('hh:nn:ss',now));
                Close(fLog);

                { the following output goes to the bit bucket }
                Writeln('daemon code activated at ',hr:0,':',mn:0,':',sc:0);
                {----------------------}
                if bTerm then
                   BREAK else
                   { wait a while }
                   fpSelect(0,nil,nil,nil,secs*1000);
          until bTerm;

          { Clean up on closing the daemon }
          if bTerm then begin
             Append(flog);
             Writeln(flog,'daemon destroyed at ',formatdatetime('hh:nn:ss',now));
             Close(fLog);
             DeletePid();
             myserver1.free;
          end;
     end else begin
        Writeln('daemon already running at: ',oldpid);
        fpKill(oldpid, SIGTERM);
     end;
End.
